unit miniini;
(*******************************************************
   Minimum ini file
   all lines are key=value with no frills
   allow ';' to indicate comments
*******************************************************)


interface

uses     Classes,useful,sysutils,forms,MessageDialogBx;

type
   TMiniIni = class(tstringlist)
     private
       FFilename : string;
       FLoaded   : boolean;
       FChanged  : boolean;
       procedure   Load;
       procedure   Save;
     public
       constructor create(const aFilename:string);
       destructor  destroy;  override;
       function    getstr(const ky:string):string;
       function    getint(const ky:string):integer;
       function    getintdef(const ky:string;const deflt:integer):integer;
       function    getDateTime(const ky:string):TDatetime;
       procedure   setstr(const ky,v:string);
       procedure   setInt(const ky:string;  const v:integer);
       procedure   setDateTime(const ky:string; const v:TDatetime);
     published
       property    name : string  read FFilename;  
     end;




implementation

constructor TMiniIni.create(const aFilename:string);
{ set up the mini-ini tolook at the specified file
  if aFilename is blank then use the application executable with extension changed to '.conf'
  if aFilename is not blank but doesn't contain a '.' then interpret
  it as an alias and use the _path_ from that plus the application _name_ plus the extension '.conf' }
var
  f : string;
begin
  inherited create;
  // work out filename
  f := afilename;
  if f = '' then begin
    f := changefileext(application.exename,'.conf');
  end else begin
    if pos('.',f)=0 then begin
      f := pathforalias(f)+'\'+extractfilename(changefileext(application.exename,'.conf'));
    end;
  end;
  FFilename := f;
  if not fileexists(f) then begin
    okbx('Warning|The configuration file "'+f+'" cannot be found');
    Floaded := true;
    Fchanged := false;
  end else begin
    Load;
  end;  
end;

destructor TMiniIni.destroy;
{ make sure it is saved if any changes have been made }
begin
  if FChanged then save;
  inherited destroy;
end;


procedure TMiniIni.load;
{ load from file given in FFilename }
begin
  if not Floaded then begin
    loadfromfile(FFilename);
    FLoaded := true;
    FChanged := false;
  end;
end;

procedure TMiniIni.save;
{ save to file given in FFilename }
begin
  if Floaded then begin
    if Fchanged then begin
      savetofile(FFilename);
      FChanged := false;
    end;
  end;
end;

function    TMiniIni.getstr(const ky:string):string;
var
  v : string;
begin
  v := trim(values[Uppercase(ky)]);
  result := trim(chop(v,';'));
end;


function    TMiniIni.getint(const ky:string):integer;
begin
  result := strtointdef(getstr(ky),0);
end;

function    TMiniIni.getintdef(const ky:string;const deflt:integer):integer;
begin
  result := getint(ky);
  if result = 0 then result := deflt;
end;



function    TMiniIni.getDateTime(const ky:string):TDatetime;
begin
  result := strtodatetime(getstr(ky));
end;


procedure   TMiniIni.setstr(const ky,v:string);
var
  s,vold,vnew : string;
begin
  s := values[Uppercase(ky)];                 // full text including any ;
  vold := trim(chop(s,';'));                  // active element
  if vold<>v then begin
    vnew := v;
    if trim(s)<>'' then vnew := vnew + '  ;'+s;       // will preserve trailing comments like '; ....'
    values[Uppercase(ky)] := vnew;
    FChanged := true;
  end;
end;


procedure   TMiniIni.setInt(const ky:string; const v:integer);
begin
  SetStr(ky,inttostr(v));
end;


procedure   TMiniIni.setDateTime(const ky:string; const v:TDatetime);
begin
  SetStr(ky,datetimetostr(v));
end;


end.
